/* sprites.h 

***************************************************************************
In deze include-file staan de sprite routines voor svgalib.
Elke sprite is 32x32 pixels in maximaal 256 kleuren. Per sprite is 2 kB
geheugen nodig; 1 kB voor de opslag van de sprite definitie en 1 kB is 
nodig voor het tijdelijk opslaan van de beeldscherm data op de positie waar
de sprite geplaatst wordt.
***************************************************************************
Auteur : Roland Leurs
Versie : 1.0
Datum  : 13 januari 2000
***************************************************************************

---------------------------------------------------------------------------
Sprite routines vanaf versie 1.0:
De volgende functies zijn geimplementeerd:
sprite_init()  - Definieer sprite base en initialisatie
sprite_def()   - Definieer sprite
sprite_set()   - Plaats sprite op het scherm
sprite_unset() - Verwijder sprite van het scherm
sprite_move()  - Verplaats sprite op het scherm
sprite_put()   - Plaats sprite op scherm, zonder administratie
sprite_hit()   - Test overlap van twee sprites

sprite_init() geeft als return-waarde terug of het alloceren van de 
benodigde geheugenruimte gelukt is. De overige functies retourneren
een statusbyte. De afzonderlijke bits hebben de volgende betekenis:

 +---+---+---+---+---+---+---+---+
 | 7 | 6 | 5 | 4 | 3 | 2 | 1 | 0 |
 +---+---+---+---+---+---+---+---+
   |   |   |   |   |   |   |   |
   |   |   |   |   |   |   |   +---- muis button 1 (alleen sprite_hit)
   |   |   |   |   |   |   +-------- muis button 2 (alleen sprite_hit)
   |   |   |   |   |   +------------ muis button 3 (alleen sprite_hit)
   |   |   |   |   +---------------- ongeldige parameter
   |   |   |   +-------------------- geen sprites beschikbaar cq niet geinitialiseerd
   |   |   +------------------------ ongeldig spritenummer 1
   |   +---------------------------- ongeldig spritenummer 2 of geen muis
   +-------------------------------- overlap tussen sprite 1 en 2 cq muis

Muis routines vanaf versie 1.0:
mouse_init()   - initialiseert de muis voor gebruik
mouse_on()     - activeert de muiscursor op het scherm
mouse_off()    - schakelt de muiscursor op het scherm uit
mouse_move()   - verplaatst de muiscursor op het scherm
mouse_hotspot()- stelt hotspot van muis in

Gebruikte globale variabelen:
sprmax        - maximaal aantal sprites
sprbase	      - pointer naar sprite administratie geheugen
sprx[]        - array met x-coordinaten
spry[]        - array met y-coordinaten
__olmx        - oude x-positie van de muiscursor
__olmy        - oude y=positie van de muiscursor
__mhsx        - x positie van hotspot
__mhsy        - y positie van hotspot

Deze sprite functies zijn afgeleid van Atom-in-PC terminal 4.0
Verspreiding en wijziging is toegestaan mits oorspronkelijke auteur duidelijk
in de source genoemd blijft.
*/

#include "graphics.h"
int MOUSECUR=0;				/* Sprite number of mouse cursor */
char *sprbase;
int sprmax, *sprx, *spry;
int __olmx=-1, __olmy=-1, __mhsx=0, __mhsy=0;

int sprite_init(int nr_of_sprites)
	{
	if(!sprbase)
		{
		int i;
		i=nr_of_sprites+1;
		sprbase = malloc(i*2048);
		sprx = malloc(i*sizeof(int));
		spry = malloc(i*sizeof(int));
		if(!sprbase || !sprx || !spry)
			return(1);			/* allocation failed */
		else
			{
			sprmax=nr_of_sprites;
			for(i=0; i<=sprmax; i++)
				sprx[i]=spry[i]=-1;
			__olmx=__olmy=-1;
			}
		}
	return(0);					/* sprites initialized */
	}

int sprite_def(int sprite_nr, char *sprdata)
	{
	int i;
        if(sprbase)
                if(sprite_nr <= sprmax)
                        {
                        for(i=0; i<1024; i++)
                                sprbase[i+2048*sprite_nr] = sprdata[i];
                        return 0;       /* No errors */
                        }
                else return 0x20;      	/* Bad sprite number */
        else return 0x10;               /* Sprites not initialized */
        }

void _spr_set(int nr, int xp, int yp)
	{
	int x,y,i,j,p;
	x=xp;
	y=yp;
	p=2048*nr;
	for(j=0; j<32; j++)
		for(i=0; i<32; i++)
			{
			if(sprbase[p]!=(char)0xFF)
				{
				vga_setcolor((int)sprbase[p]);
				if((x+i)<=X__max && (y+j)<=Y__max)
					vga_drawpixel(x+i, y+j);
				}
			p++;
			}
	}

int sprite_set(int sprite_nr, int x, int y)
        {
        int i;
	char *p;
        if(sprbase)
                if(sprite_nr <= sprmax)
                        {
			p=sprbase+1024+2048*sprite_nr;
			for(i=0; i<32; i++)
				if(y+i<=Y__max)
					vga_getscansegment(p+i*32, x, y+i, 32);
			_spr_set(sprite_nr, x, y);
			sprx[sprite_nr]=x;
			spry[sprite_nr]=y;
                        return 0;       /* No errors */
                        }
                else return 0x20;       /* Bad sprite number */
        else return 0x10;               /* Sprites not initialized */
        }

int sprite_unset(int sprite_nr)
        {
        int i;
	char *p;
        if(sprbase)
                if(sprite_nr <= sprmax)
                        {
                        p=sprbase+1024+2048*sprite_nr;
                        for(i=0; i<32; i++)
				if(spry[sprite_nr]+i<=Y__max)
                                	vga_drawscansegment(p+i*32, sprx[sprite_nr], spry[sprite_nr]+i, 32);
                        sprx[sprite_nr]=spry[sprite_nr]=-1;
                        return 0;       /* No errors */
                        }
                else return 0x20;       /* Bad sprite number */
        else return 0x10;               /* Sprites not initialized */
        }
 
int sprite_move(int sprite_nr, int new_x, int new_y)
        {
        if(sprbase)
                if(sprite_nr <= sprmax)
                        {
			sprite_unset(sprite_nr);
			sprite_set(sprite_nr, new_x, new_y);
                        return 0;       /* No errors */
                        }
                else return 0x20;       /* Bad sprite number */
        else return 0x10;               /* Sprites not initialized */
        }
 
int sprite_put(int sprite_nr, int x, int y)
        {
        if(sprbase)
                if(sprite_nr <= sprmax)
                        {
                        _spr_set(sprite_nr, x, y);
                        return 0;       /* No errors */
                        }
                else return 0x20;       /* Bad sprite number */
        else return 0x10;               /* Sprites not initialized */
        }

int sprite_hit(int sprite1, int sprite2)
	{
	int status=0;
	if(sprbase)
		{
		if(sprite1 > sprmax) status|=0x20;
		if(sprite2 > sprmax) status|=0x40;
		if(!status)
			{
			if(sprite2==MOUSECUR)
				{
				mouse_update();
				if(__mhsx+sprx[sprite2]>=sprx[sprite1] && __mhsx+sprx[sprite2]<sprx[sprite1]+32 && __mhsy+spry[sprite2]>=spry[sprite1] && __mhsy+spry[sprite2]<spry[sprite1]+32) status=0x80;
	                        status+=(mouse_getbutton() & 7);
				}
			else
				if((abs(sprx[sprite1]-sprx[sprite2])<32) && (abs(spry[sprite1]-spry[sprite2])<32)) status=0x80;
			}
		return status;
		}
	else return 0x10;		/* Sprites not initialized */
	}

int mouse_on(void)
	{
	if(sprbase)
		{
		mouse_update();
                sprite_set(MOUSECUR, mouse_getx(), mouse_gety());
                return 0;       	/* No errors */
                }
        else return 0x10;               /* Sprites not initialized */
	}

int mouse_move(void)
	{
	int x,y;
        if(sprbase)
                {
                mouse_update();
                x=mouse_getx();
		y=mouse_gety();
		if(x!=__olmx || y!=__olmy)
			{
			sprite_move(MOUSECUR, x, y);
			__olmx=x;
			__olmy=y;
			}
                return 0;       	/* No errors */
                }
        else return 0x10;               /* Sprites not initialized */
        }

int mouse_off(void)
        {
        if(sprbase)
                {
                sprite_unset(MOUSECUR);
                __olmx=__olmy=-1;
                return 0;       	/* No errors */
                }
        else return 0x10;               /* Sprites not initialized */
        }

int mouse_hotspot(int x, int y)
        {
        if(sprbase)
                {
		if(x<0 || x>31 || y<0 || y>31)
			return 0x08;
		else
			{
			__mhsx=x;
			__mhsy=y;
	                return 0;       /* No errors */
			}
                }
        else return 0x10;               /* Sprites not initialized */
        }

